﻿import mx.utils.Delegate;

class Midi {
	
	var server_ip 		= "127.0.0.1";
	var server_port		= 10002;
	
	var varDelimiter 	= "/;/";
	var keyDelimiter	= "/,/";
	var reqDelimiter	= "/#/";
	
	var serialSocket;
	var _reqID			= 0;
	var _requests		= {};
	var _connected		= false;
	
	var _onNoteOn;
	var _onNoteOff;
	var _onNoteChange;
	var _onError;
	var _onConnect;
	
	var meMC;
	
	var _playIntervalIDs 	= {};
	var _intervalCnt		= 0;
	var _connectInterval;
	
	var _myInput;
	var _myOutput;
	
	function get_devices ( func ) {
		send_request("get_list_midi_devices", {}, func);
	}
	
	function set input ( name ) {
		send_request("set_midi_input", {name:name});
	}
	
	function set output ( name ) {
		send_request("set_midi_output", {name:name});
	}

	function setDevices ( input, output ) {
		send_request("set_midi_devices", {name_input:input, name_output:output});
	}

	function set onNoteOn ( func ) {
		_onNoteOn = func;
	}

	function set onNoteOff ( func ) {
		_onNoteOff = func;
	}
	
	function set onNoteChange ( func ) {
		_onNoteChange = func;
	}
	
	function set onError ( func ) {
		_onError = func;
	}
	
	function set onConnected ( func ) {
		_onConnect = func;
	}
	
	// ----------------------------------------------------------------------------
	
	function Midi ( input, output ) {
		meMC = this;
		_myInput = input ? input : "all";
		_myOutput = output ? output : _myInput;
		setDevices( _myInput, _myOutput );
		init();
	}
	
	function init () {
		serialSocket = new XMLSocket();		
		serialSocket.onConnect 	= Delegate.create( meMC, on_connect );
		serialSocket.onClose	= Delegate.create( meMC, try_reconnect );
		serialSocket.onData 	= Delegate.create( meMC, on_data );
		serialSocket.connect( server_ip, server_port );
	}
	
	function on_connect ( success ) {
		clearInterval( _connectInterval );
		_connected = success;
		//trace( success ? "Verbindung zu " + server_ip + " erfolgreich" : "Keine Verbindung zu Java. Erneuter Versuch in 1 sek.");
		if (_connected) {
			_onConnect( 1, "midimicrobe gefunden!" );
			send_buffered_requests();
		} else {
			try_reconnect();
		}
	}

	function send_error( id, message ) {
		_onError( id, message );
	}
	
	
	function try_reconnect () {
		send_error( -1, "Keine Verbindung zur midimicrobe." );
		_connectInterval = setInterval( this, "init", 1000 );
	}
	
	function on_data ( str ) {
		var p = parse_answer( str );

		if (p.id == 0) {
			//trace(p.type.toString(2));

			if (p.type&16 && !p.velo) {
				_onNoteOff.apply( null, p.obj );
			}
			else if (p.type&32) {
				_onNoteChange.apply( null, p.obj );
			}
			else if (p.type&16) {
				_onNoteOn.apply( null, p.obj );
			} 
			else if (!(p.type&16)) {
				_onNoteOff.apply( null, p.obj );
			}
			return false;
		}
		
		if (_requests[p.id]) {
			_requests[p.id].func.apply( null, p.obj );
			delete( _requests[p[0]] );
		}
	}

	// ----------------------------------------------------------------------------

	function sendNoteOn ( channel, note, velo ) {
		if (velo == undefined) { velo = note; note = channel; channel = 0; }
		send_request("send_note_on", {channel:channel, note:note, velo:velo});
	}
	
	function sendNoteOff ( channel, note, velo ) {
		if (velo == undefined) { velo = note; note = channel; channel = 0; }
		send_request("send_note_off", {channel:channel, note:note, velo:velo});
	}

	function playNote ( channel, note, velo, duration ) {
		if (duration == undefined) { duration = velo; velo = note; note = channel; channel = 0; }
		_intervalCnt++;
		_intervalCnt = _intervalCnt%9999;
		_playIntervalIDs[_intervalCnt] = setInterval( meMC, "playNoteOver", duration, {id:_intervalCnt, channel:channel, note:note, velo:velo} );
		sendNoteOn( channel, note, velo );
	}
	
	function playNoteOver ( obj ) {
		clearInterval( _playIntervalIDs[obj.id] );
		sendNoteOff( obj.channel, obj.note, obj.velo );
	}

	
	// ----------------------------------------------------------------------------

	function send_request ( action, reqVars, func, id ) {
		if (id == undefined) {
			id = create_request_id();
			_requests[id] = {id:id, vars:reqVars, action:action, func:func};
		}
		if (_connected) {
			var reqStr = build_request( id, action, reqVars);
			serialSocket.send( reqStr );
			//trace("Anfrage gesendet: " + reqStr);
		}
	}
	
	function send_buffered_requests () {
		var arr = [];
		for (var i in _requests) arr.push( _requests[i] );
		for (var i in arr) send_request( arr[i].action, arr[i].vars, arr[i].func, arr[i].id );

	}

	function build_request ( id, action, reqVars ) {
		var arr = [];
		for (var i in reqVars) arr.push( i + keyDelimiter + reqVars[i] );
		return id + varDelimiter + action + varDelimiter + arr.join( varDelimiter ) + reqDelimiter;
	}
	
	function create_request_id () {
		_reqID = (++_reqID)%9999;
		return _reqID + 1;
	}

	// ----------------------------------------------------------------------------

	function parse_answer ( str ) {
		
		var p 	= str.split( varDelimiter );
		var id 	= p[0];
		
		// Midi-Signal angekommen
		if (id == 0) return {id:id, obj:[p[1], p[2], p[3]], type:p[3]*1, velo:p[2]*1, note:p[1]*1};
		
		var req = _requests[id];
		
		switch (req.action) {
			case "get_list_midi_devices":
				return {id:id, obj:[p[1].split(keyDelimiter), p[2].split(keyDelimiter)]}
				break;
		}
	}

}